import ij.*;
import ij.plugin.PlugIn;

import mpicbg.models.*;

import net.imglib2.*;
import net.imglib2.img.*;
import net.imglib2.img.display.imagej.ImageJFunctions;
import net.imglib2.util.Util;
import net.imglib2.interpolation.randomaccess.*;
import net.imglib2.type.*;
import net.imglib2.type.numeric.*;
import net.imglib2.type.numeric.real.*;
import net.imglib2.view.*;

/**
 *  Transform an image in N dimensions using Linear interpolation
 *  and OutOfBoundsStrategies
 */
public class ImgLib2_Transform implements PlugIn
{
	public void run(String arg0)
	{
		// get the current ImageJ ImagePlus
		ImagePlus imp = WindowManager.getCurrentImage();

		// test if an image is open
		if ( imp == null )
		{
			IJ.log( "No image open" );
			return;
		}

		// wrap it into an ImgLib2 Img (no copying)
		Img<RealType> img = ImageJFunctions.wrapReal( imp );

		// test if it could be wrapped
		if ( img == null )
		{
			IJ.log( "Cannot wrap image" );
			return;
		}

		// process wrapped image with ImgLib2
		process( img );
	}

	public <T extends RealType<T>> void process( Img<T> img )
	{
		// Define the transformation model
		RigidModel2D model = new RigidModel2D();
		model.set( (float)Math.toRadians( 15 ),  0, 0 );

		// Define the transformation model
		//TranslationModel2D model = new TranslationModel2D();
		//model.set( 10.1f, -12.34f );

		try
		{
			// compute the gradient on the image
			Img<T> transformed = transform( img, model );
	
			// show the new Img that contains the gradient
			ImageJFunctions.show( transformed );
		}
		catch ( NoninvertibleModelException e )
		{
			IJ.log( model + " cannot be inverted: " + e );
		}
	}

	public <T extends RealType<T>> Img<T> transform( Img< T > img, InvertibleBoundable transform ) throws NoninvertibleModelException
	{
		// create a new ImgLib2 image of same type & dimensions
		Img<T> transformedImg = ...
				
		// create a localizing cursor on the TransformedImg, it will iterate all pixels
		// and is able to efficiently return its position at each pixel. At each pixel
		// we will look up the intensity at the corresponding, transformed location in
		// the input image
		Cursor<T> cursor = ...
		
		// We extend the input image by a value out of bounds strategy so
		// that we can access pixels outside of the image

		// create a value background by grabbing the first element from img and making
		// a copy of it
		T background = ...

		// now set its value to the desired value, e.g. 0
		background.

		// create an infinite View on the img by extending it
		RandomAccessible<T> view = Views.extendValue( img, background );

		// We create a RealRandomAccessible by linear interpolating the view, i.e. we can 
		// compute a pixel value at any floating point location (e.g. 4.5, 5.1, ...)
	        NLinearInterpolatorFactory< T > factory = new NLinearInterpolatorFactory< T >();
		RealRandomAccessible< T > interpolant = Views.interpolate( view, factory );

		// instantiate a RealRandomAccess on the extended, interpolated view
		RealRandomAccess< T > realRandomAccess = ...

		// a temporary array to compute the transformation
		float[] tmp = new float[ img.numDimensions() ];
	
		// iterate over all pixels
		while ( cursor.hasNext() )
		{
			// move the cursor to the next pixel
			
			// get the location of the cursor
			
			// apply the inverse transformation to the temporary array
			
			// set the interpolator to the transformed location
			
			// set the value of the interpolator to the cursor
		}

		return transformedImg;
	}
}